# -----------------------------------------------------------------------------
package AO_Config;
# -----------------------------------------------------------------------------

use strict;

# -----------------------------------------------------------------------------
# -- CONSTRUCTOR :
# -----------------------------------------------------------------------------

sub new() {

  my $class = shift;
  my $devel = shift;

    my $self = { };

    bless($self, $class);

    $self->{DEVEL} = $devel;

    return $self;
  }


# -----------------------------------------------------------------------------
# -- Error messages :
# -----------------------------------------------------------------------------

sub dave() {
    return "I'm sorry, Dave.  I'm afraid I can't do that. "
  }

# -----------------------------------------------------------------------------
# -- YP/NIS, Solaris or Flat Files, Linux ?  Choose appropriate modules...
# -----------------------------------------------------------------------------

sub Auto_Home() {
    return "YP_Sol_Auto_Home";
##  return "Etc_Lin_Auto_Home";
  }

sub Passwd() {
    return "YP_Sol_Passwd";
##  return "Etc_Lin_Passwd";
  }

sub Passwd_Adj() {
    return "YP_Sol_Passwd_Adj";
##  return "Etc_Lin_Passwd_Adj";
  }

# -----------------------------------------------------------------------------
# -- LOGS : 
# -----------------------------------------------------------------------------

sub log_file() {
    my $self = shift;
    return "/home/mpciish2/var/log/admin_objects";
  }

sub acc_cr_log_file() {
    my $self = shift;
    return "/home/mpciish2/var/log/admin_objects_acc_cr";
  }


# -----------------------------------------------------------------------------
# -- ETC ON THIS MACHINE :
# -----------------------------------------------------------------------------

sub etc_group() {
    my $self = shift;
    return "/etc/group";
  }

# -----------------------------------------------------------------------------
# -- NEW ACCOUNT SPECIFIC STUFF :
# -----------------------------------------------------------------------------

sub new_account_message() {
    my $self = shift;

    my $message 
        = "\n "
        . "\n This message is automatically generated by the Cosmos"
        . "\n account-creation programme (Admin_Objects)."
        . "\n "
        . "\n A Cosmos (unix/Solaris) account has been set up for you."
        . "\n The username is the same as your eUMIST username."
        . "\n "
        . "\n If you have not been informed of your password or have forgotten"
        . "\n it please visit the ISD Helpdesk in B28, Renold building "
        . "\n (please bring you ID/Library Card with you)."
        . "\n "
        . "\n Details of the Cosmos Unix Service, including applications"
        . "\n available and printing queues can be found at"
        . "\n "
        . "\n     http://www.umist.ac.uk/isd/unix    "
        . "\n ";

    return $message;
  }

# -----------------------------------------------------------------------------
# -- WHERE ARE THE HOME DIRS?
# -----------------------------------------------------------------------------

my @home_dirs = ("/export/u01", "/export/u02", "/export/u03", 
                 "/export/u04", "/export/u05", "/export/u06", 
                 "/export/u07", "/export/u08", "/export/u09");

# -- directory we are currently using for new home directories :

sub new_home_dir_partition() {
    my $self = shift;
                              
    # -- Cosmos :
    return "/export/u06";     ### --- needs doing systematically somehow
                              ### --- needs doing systematically somehow
                              #       e.g., the most empty of u06, 7 
                              #             and 8...

    # -- devel :
    #return "/home/mpciish2/src/_cosmos_admin_objects_v0.2/_export/u06";
  }

# -----------------------------------------------------------------------------
# --

sub home_dirs() {
    my $self = shift;
    return \@home_dirs;
  }

# -----------------------------------------------------------------------------
# --

sub export() {
    my $self = shift;

    # -- Solaris :
    return "/export";

    # -- Etc-Linux flat files (relevant?) :
    #return "/home";

    # -- devel :
    #return "/home/mpciish2/src/_cosmos_admin_objects_v0.2/_export";
  }


# -----------------------------------------------------------------------------
# -- INTERFACE :
# -----------------------------------------------------------------------------

sub gui() {
    my $gui = "Txt";          # --  "Txt", "X11", "www"...;
    return $gui;
  }

# -----------------------------------------------------------------------------
# -- WHERE ARE THE YP FILES ?
# -----------------------------------------------------------------------------

# -- the directory holding YP's passwd, or the password (flat) 
#    file, or whatever (the thing with the usernames, home-dir, shell, etc) :

sub passwd_dir() {
    my $self = shift;

    # -- Solaris YP :
    return "/var/yp/ypfiles";

    # -- Etc flat files :
    #return "/etc";

    # -- devel :
    #return "/home/mpciish2/src/_cosmos_admin_objects_v0.2/_var/yp/ypfiles";
  }

# -- the directory holding YP's passwd.adjunct, or the password shadow
#    file, or whatever (the thing with the encrypted passwords in) :

sub passwd_sec_dir() {
    my $self = shift;

    # -- Solaris YP :
    return "/var/yp/ypfiles/security";

    # -- Etc flat files :
    #return "/etc";

    # -- devel :
    #return "/home/mpciish2/src/_cosmos_admin_objects_v0.2/" 
    #     . "_var/yp/ypfiles/security";
  }

# -- the password file itself :

sub passwd() {
    my $self = shift;

    # -- Solaris YP :
    return "/var/yp/ypfiles/passwd";

    # -- Etc flat files :
    #return "/etc/passwd";

    # -- devel :
    #return "/home/mpciish2/src/_cosmos_admin_objects_v0.2/" 
    #     . "_var/yp/ypfiles/passwd";
  }

# -- the file with the encrypted passwords, itself :

sub passwd_adj() {
    my $self = shift;

    # -- Solaris YP :
    return "/var/yp/ypfiles/security/passwd.adjunct";

    # -- Etc flat files :
    #return "/etc/shadow";

    # -- devel :
    #return "/home/mpciish2/src/_cosmos_admin_objects_v0.2/" 
    #     . "_var/yp/ypfiles/security/passwd.adjunct";
  }

sub auto_home() {
    my $self = shift;

    # -- Solaris YP :
    return "/var/yp/ypfiles/auto_home";

    # -- Etc flat files :
    #return "/etc";

    # -- devel :
##  return "/home/mpciish2/src/_cosmos_admin_objects_v0.2/_var/yp/ypfiles/auto_home";
  }

# -----------------------------------------------------------------------------
# -- MACHINE-SPECIFIC STUFF :
# -----------------------------------------------------------------------------

sub hostname() {
    my $self = shift;
    return "cosmos";
  }

sub scratch() {
    my $self = shift;
    return "/scratch";
  }

sub var_mail() {
    my $self = shift;
    return "/var/mail";
  }

# -----------------------------------------------------------------------------
# -- UMIST-SPECIFIC STUFF :
# -----------------------------------------------------------------------------

my $grp_name_2_gid_map 
    = { "mjf" => "1262",
        "mce" => "148",
        "mjk" => "258",
        "mcd" => "140",
        "mcg" => "243",
        "mca" => "137",
        "mjh" => "901",
        "mjp" => "902",
        "mcn" => "128",
        "mch" => "143",
        "mpq" => "903",
        "mcu" => "142",
        "mcy" => "248",
        "mcl" => "1374",
        "mbl" => "147",
        "mcb" => "138",
        "mcj" => "144",
        "mjc" => "904",
        "mcf" => "117",
        "mcc" => "139",
        "mjr" => "124",
        "mps" => "905",
        "mpl" => "1373",
        "mpr" => "132",
        "mct" => "136",
        "mcr" => "129",
        "mpc" => "1226",
        "mph" => "906" 
      };

# -- return group number (gid) from three-letter string, e.g., mpc :

sub gid() {
    my $self   = shift;
    my $string = shift;

    my $str = substr($string, 0, 3);
    
    $self->{DEVEL}->output("Determining gid based on: $str");

    if (defined $grp_name_2_gid_map->{$str}) {
        return $grp_name_2_gid_map->{$str};
      }
    else {
        die "\n\n Config::gid :  unrecognised string :  $str \n\n";
      }
  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
