package AO_New_Account;

use strict;
use vars qw(@ISA);

# -- my stuff :

use Home_Dirs;
use Mail;

# -- CPAN stuff :

use Net::LDAP;
use Quota;

# -----------------------------------------------------------------------------

@ISA = ("AO_Template");

# -----------------------------------------------------------------------------

my $menu_string = "\n " 
                . "\n New Cosmos Account"
                . "\n ------------------"
                . "\n "
                . "\n Enter eUMIST username (blank to quit), <return> : ";


# -----------------------------------------------------------------------------

sub new() {

  my $class      = shift;
  my $devel      = shift;
  my $logs       = shift;
  my $menus      = shift;
  my $yp_passwd  = shift;
  my $config     = shift;
  my $unix_time  = shift;
  my $human_time = shift;

    my $self = { };

    bless($self, $class);

    $self->{MENU_STR} = $menu_string;

    $logs->append_new_acc("\n\nAO_New_Accounts::new : \t" . $unix_time
                        . " \t("                     . $ENV{LOGNAME} . ")"
                        . " \t" . $human_time);

    # -------------------------------------------------------------------------
    # -- Load appropriate YP/NIS+/Flat thingy :

    my $auto_home_pm  = $config->Auto_Home();
    my $passwd_adj_pm = $config->Passwd_Adj();

    require "$auto_home_pm.pm";
    require "$passwd_adj_pm.pm";
                            
    $self->{AUTO_HOME} 
           = $auto_home_pm->new($config->auto_home, $config, $devel, $logs);
    $self->{PASSWD_ADJ} 
           = $passwd_adj_pm->new($config->passwd_adj, $config, $devel, $logs);

    # -------------------------------------------------------------------------

    $self->new_AO_template($devel, $logs, $menus, $yp_passwd, $config);

    return $self;
  }

# -----------------------------------------------------------------------------
# -- THIS IS A SPECIAL MODULE --- SUB DO_IT IS A LITTLE DIFFERENT FROM THE
#    OTHERS AS ALL OTHERS DEAL WITH AN EXISTING ACCOUNT, WE DONT!

sub do_it() {
    my $self = shift;
    my $username = shift;

    $self->do_it_init($username);

    # -------------------------------------------------------------------------
    # -- STANDARD CHECKS :

##  my ($one, $two) = $self->do_it_check($username);
##  if ($one) {return $two}

        # ...not with this module --- can't check an account not yet 
        #    in existence!

    unless ($username =~ m/./) {return ("done_it")}


    # -------------------------------------------------------------------------
    # -- MODULE-SPECIFIC STUFF :

    my $common_name = "";
    my $email_addr  = "";

    # -- ensure eUMIST username given is actually an eUMIST username, if
    #    so display details for confirmation :

    $self->{LOGS}->append("Searching for eUMIST account :  $username");
    $self->ldap_connect_anon();

    my ($one, $entry, $attributes) = $self->ldap_search($username);

    # -- if we've got something back continue, else advise admin and return :

    unless ($one) {
        $self->{LOGS}->append("Could not find LDAP entry for: $username");
        $self->{MENUS}->message($entry); 
        return "";
      }

    # -- display some of the info we got back to confirm we have the 
    #    right person :

    foreach my $a (@{$attributes}) { 
        my @att_vals = $entry->get_value($a);

        if (($a eq "cn") || ($a eq "sn") || ($a eq "mail")) {

            if ($a eq "cn")   { $common_name = $att_vals[0]; }
            if ($a eq "mail") { $email_addr  = $att_vals[0]; }

            $self->{LOGS}->append("$a \t : ");
            $self->{MENUS}->append_ln("$a \t : ");
            foreach my $v (@att_vals) {
                $self->{LOGS}->append("$v, ");
                $self->{MENUS}->append("$v, ");
              }
          }
      }

    # -- confirm we're going ahead :

    unless ($self->{MENUS}->confirm("Create account for this person?")) {
        return "";
      }

    # -- does this person already have a Cosmos account?

    if ($self->already_has_account($username)) {
        $self->{LOGS}->append("Already has account: $username");
        return "";
      }
       
    # -- all appears well :  create account...

    $self->{LOGS}->append("Attempting to creating new account, " 
                        . "eUMIST username: $username");

    my ($two, $passwd, $uid, $gid) 
                              = $self->create_account($username, $common_name);

    unless ($two) {
        $self->{LOGS}->append("UNABLE TO CREATE ACCOUNT: $username");
        $self->{MENUS}->message("Unable to create account --- TELL SIMON!!");
        return "";
      }

    # ...create filespaces... :
    my ($three, $three_str) = $self->create_filespaces($username, $uid, $gid);

    unless ($three) {
        $self->{LOGS}->append("ERROR : UNABLE TO CREATE FILESPACE: " 
                            . "$username --- $three_str");
        $self->{MENUS}->message("ERROR : " 
                              . "Unable to create filespace --- TELL SIMON!!");
        return "";
      }

    # ...initialise quotas... :
    my ($four, $four_str) = $self->initialise_quotas($username, $uid);

    unless ($four) {
        $self->{LOGS}->append("ERROR : UNABLE TO INIT QUOTAS: " 
                            . "$username --- $four_str");
        $self->{MENUS}->message("Unable to init quotas --- TELL SIMON!!");
        return "";
      }

    # ...and email new account holder... :
    my ($five, $five_str) = $self->mail_new_acc_holder($username, $email_addr);

    unless ($five) {
        $self->{LOGS}->append("UNABLE TO EMAIL NEW ACC HOLDER: " 
                            . "$username --- $five_str");
        $self->{MENUS}->message("Unable to email new acc holder " 
                              . "--- TELL SIMON!!");
        return "";
      }

    # -- account-log the event and tell the admin :

    $self->{LOGS}->append_new_acc("Account created ok: $username\n\n");
    $self->{LOGS}->append("Account created ok: $username");
##  $self->{MENUS}->message("Account created and active, password:" . $passwd);
    $self->{MENUS}->message("Account created and active!");

    return "done_it";
  }


# -----------------------------------------------------------------------------
# -- CREATE ACCOUNT :
# -----------------------------------------------------------------------------

sub already_has_account() {
    my $self     = shift;
    my $username = shift;

    $self->{MENUS}->append_ln("Looking for evidence of existing account...");

    $self->{MENUS}->append_ln();
    $self->{MENUS}->append_b_ln("Looking in passwd space...");
    $self->{MENUS}->append_ln();

    my $yp_p_matches = $self->{PASSWD}->grep_users($username);
    foreach (@$yp_p_matches) {print "\n $_"}

    
    $self->{MENUS}->append_ln();
    $self->{MENUS}->append_b_ln("Looking in passwd.adjunct space...");
    $self->{MENUS}->append_ln();

    my $yp_p_adj_matches = $self->{PASSWD_ADJ}->grep_users($username);
    foreach (@$yp_p_adj_matches) {print "\n $_"}

    return $self->{MENUS}->confirm("In you expert opinion, does the user " 
                                 . "already have an account?");
  }


sub create_account() {
    my $self     = shift;
    my $username = shift;
    my $commname = shift;

    my $passwd = "the_passwd";
    my $config = $self->{CONFIG};

    my ($one, $uid, $gid)  = $self->{PASSWD}->new_account($username, 
                                                             $commname);
    my ($two, $txt_passwd) = $self->{PASSWD_ADJ}->new_account($username);
    my ($three)            = $self->{AUTO_HOME}->new_account($username);

    unless ($one && $two && $three) {
        return (0, "");
      }

    my $four = $self->{PASSWD}->append_new_acc($username);
    my $five = $self->{PASSWD_ADJ}->append_new_acc($username);
    my $six  = $self->{AUTO_HOME}->append_new_acc($username);

    # ...update NIS (ascii files already updated, simply run the make file... :
    $self->{MENUS}->append_ln("");
    $self->{MENUS}->append_ln("Messages below are from the NIS Makefile... :");
    $self->{MENUS}->append_ln("---------------------------------------------");
    $self->{MENUS}->append_ln("\n");

    my $pwd = `/usr/bin/pwd`;
    chdir("/var/yp");
    system("/usr/ccs/bin/make");

    $self->{MENUS}->append_ln("");
    $self->{MENUS}->append_ln("------ End messages from NIS Makefile -------");
    $self->{MENUS}->append_ln("");
    $self->{MENUS}->append_ln("");

    unless ($pwd =~ m/^([a-zA-Z0-9\.\_\-\/]+)$/) {
        $self->{DEVEL}->die("Tainted: $pwd");
      }
    $pwd = $1;

    chdir($pwd);

    return (1, $txt_passwd, $uid, $gid);
  }


sub create_filespaces() {
    my $self      = shift;
    my $username  = shift;
    my $uid       = shift;
    my $gid       = shift;

    my $home_dirs = Home_Dirs->new($self->{CONFIG}, $self->{DEVEL}, 
                                   $self->{LOGS});

    my ($flag, $string) = $home_dirs->mk_new_home_dir($username, $uid, $gid);

    return ($flag, $string);
  }


sub initialise_quotas() {
    my $self  = shift;
    my $uname = shift;
    my $uid   = shift;

    $self->{DEVEL}->marker("username: $uname");

    my $ut_uname = $uname;

    unless ($ut_uname =~ m/^(m[a-z0-9]+)$/) {
        $self->{DEVEL}->die("Tainted: $ut_uname");
      }
    $ut_uname = $1;

#    $self->{DEVEL}->output("/usr/sbin/edquota -p mpciibw $ut_uname");

#    system("/usr/sbin/edquota -p mpciibw $ut_uname") 
#        && $self->{DEVEL}->die("Can't: /usr/sbin/edquota " 
#                             . "-p mpciibw $ut_uname");

    $self->{DEVEL}->output("Quota::setqlim: " 
                         . $self->{CONFIG}->new_home_dir_partition());

    Quota::setqlim($self->{CONFIG}->new_home_dir_partition(), $uid, 
                   50000, 55000, 0, 0,
                   0);

    $self->{DEVEL}->output("Quota::setqlim: " . $self->{CONFIG}->scratch());

    Quota::setqlim($self->{CONFIG}->scratch(), $uid, 
                   1000000, 2000000, 0, 0,
                   0);

    $self->{DEVEL}->output("Quota::setqlim: " . $self->{CONFIG}->var_mail());

    Quota::setqlim($self->{CONFIG}->var_mail(), $uid, 
                   20000, 22000, 0, 0,
                   0);

    return (1, "");
  }


sub mail_new_acc_holder() {
    my $self  = shift;
    my $uname = shift;
    my $email = shift;

    $self->{DEVEL}->marker("username: $uname");
    $self->{DEVEL}->output("mailing........");
    $self->{MENUS}->append_ln("Emailing new-account holder...");

    my $mail = Mail->new($self->{DEVEL}, $self->{CONFIG});

    my @to  = ($email);
    my @cc  = ();
    my @bcc = ("simon.hood\@umist.ac.uk");

    my $subject = "New Account: " . $self->{CONFIG}->hostname;

    my $message = $self->{CONFIG}->new_account_message();

    $mail->send(\@to, \@cc, \@bcc, $subject, $message);

    return (1, "");
  }


# -----------------------------------------------------------------------------
# -- LDAP :
# -----------------------------------------------------------------------------


sub ldap_connect_anon{

    my $self = shift;
    my $ldap = Net::LDAP->new('ldap.umist.ac.uk');

    # -- anonymous:
    ##my $ret_mesg = $ldap->bind();

    my $ret_mesg = $ldap->bind('cn=manager,o=umist,c=gb', 
                               password=>'gent.ham');

    # -- attempt to authenticate/bind anonymously;  if fails (return code
    #    greater or equal to 1) give up...
    if ($ret_mesg->code()) {
        die ("\n\n  Failed to bind to LDAP server. \n\n")
      }

    $self->{LDAP} = $ldap;
  }

# -----------------------------------------------------------------------------
# --

sub ldap_search () {

  my $self = shift;
  my $uid  = shift;

    my $ret_mesg = $self->{LDAP}->search(base=>'o=umist,c=gb',
                                  scope=>'sub',
                                  filter=>"&(objectClass=person)(uid=$uid)");

    # -- if problem talking to LDAP server :
    if ($ret_mesg->code()) {
        return (0, "Problem talking to LDAP server.", ()); 
      }
    else {
        my $entry = $ret_mesg->shift_entry();

        # -- if got something useful back :
        if (defined $entry) {
            my @attributes = $entry->attributes();
            return (1, $entry, \@attributes);
          }
        else {
            return (0, "No eUMIST entry could be found.", ()); 
          }
      }
  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
