package Admin_Objects;

use strict;
use Etc_Group;

# -----------------------------------------------------------------------------
#-- CONFIG :

my %options = (1 => ["User_Info", 
                     "Display user's account information"],

               2 => ["Grep_Account", 
                     "Grep for account-related stuff"],

               3 => ["Set_Quota", 
                     "Change user's disk-space quota"],

               4 => ["New_Account",
                     "Request new account"],

               5 => ["Dis_Account",
                     "Disable account"]);


# -----------------------------------------------------------------------------
# --

sub new() {

  my $class  = shift;
  my $devel  = shift;
  my $logs   = shift;
  my $menus  = shift;
  my $passwd = shift;
  my $config = shift;

    my $self  = { };

    bless($self, $class);

    $self->{DEVEL}  = $devel;
    $self->{LOGS}   = $logs;
    $self->{MENUS}  = $menus;
    $self->{PASSWD} = $passwd;
    $self->{CONFIG} = $config;

    $self->{MENU_MAIN_OPTS} = \%options;

    # -------------------------------------------------------------------------
    # -- WHO AM I ?

    $self->{E_UID} = $>;
    $self->{R_UID} = $<;

    if ($self->{R_UID} == 0) {
	$self->{R_UNAME} = "root";
      }
    else {
        $self->{R_UNAME} = $passwd->uname($<)
      }

    # -------------------------------------------------------------------------
    # -- DEBUG?

    if ($devel->debug()) {
        print "\n  Real uid of this process is      :  " . $self->{R_UID};
        print "\n  Effective uid of this process is :  " . $self->{E_UID};
        print "\n  Real user is                     :  " . $self->{R_UNAME};
        $menus->AK_to_Cont_Wait();
      }

    # -------------------------------------------------------------------------
    # -- AUTHORISATION?

    # -- First and foremost, am I allowed to run this script.  To do this
    #    check out uid (use $<, one of Perl's Global Special Variables) :

        # ...Nick's script checks that $ENV{LOGNAME} is a user in the
        #    group userreg (check /etc/group to see members of said group).

        # ...but this can be faked (export LOGNAME=fred, for example).
    

    # ...am I in group userreg?  Unless I am in said group, log what 
    #    happened and die :

    my $groups = Etc_Group->new($devel, $config);

    unless (($groups->member("userreg", $self->{R_UNAME})) || 
            ($self->{R_UNAME} eq "root")                 ) {
        $self->{LOGS}->append("Exception --- unauthorised user: " 
                                                           . $self->{R_UNAME});
        $self->{DEVEL}->die("Exception --- unauthorised user: "
                                                           . $self->{R_UNAME});
      }


    # -------------------------------------------------------------------------
    # -- LOG...

    $self->{UNIXTIME} = time;

    my @human_time = localtime($self->{UNIXTIME});

    $human_time[5] = $human_time[5] + 1900;
    $human_time[4] = $human_time[4] + 1;
  
    $self->{HUMANTIME} 
        = sprintf("%04d/%02d/%02d %02d:%02d:%02d", 
                  $human_time[5], $human_time[4], $human_time[3],
                  $human_time[2], $human_time[1], $human_time[0]);

    $self->{LOGS}->append("\n\nAdmin_Objects::new : \t" . $self->{UNIXTIME} 
                        . " \t"                     . $self->{R_UNAME}
                        . " \t("                     . $ENV{LOGNAME} . ")"
                        . " \t" . $self->{HUMANTIME});

    

    # -------------------------------------------------------------------------
    # -- CLEAN UP ENV --- WE MIGHT BE SUID :

    # ...and recall we are running "-T"...

    $ENV{PATH} = "/usr/bin";
    $ENV{CDPATH} = "";


    # -------------------------------------------------------------------------

    return $self;
  }


# -----------------------------------------------------------------------------
#--

sub call_AO_module() {

  my $self   = shift;
  my $passwd = shift;
  my $option = shift;

    if ($option eq "exit") {
        $self->exit();
      }
    elsif (-e "AO_$option.pm") {

        $self->{MENUS}->append_ln("...loading module: $option...");
        require "AO_$option.pm";
        $self->{MENUS}->append_ln("...required module: " 
                                . "$option, successfully...");

        my $module = "AO_$option"->new($self->{DEVEL},
                                       $self->{LOGS}, 
                                       $self->{MENUS}, 
                                       $self->{PASSWD},
                                       $self->{CONFIG},
                                       $self->{UNIXTIME}, $self->{HUMANTIME});
      } 
    else {
        $self->{MENUS}->message($self->{CONFIG}->dave());
      }
  }

# -----------------------------------------------------------------------------
# --

sub exit() {
    my $self = shift;

    $self->{LOGS}->append(caller() . ": Exiting normally.");
    $self->{LOGS}->append("\n\n");

    print "\n ++";
    print "\n ++ Actions have been logged in " . $self->{CONFIG}->log_file();
    print "\n ++ The End";
    print "\n ++";
    print "\n";
    exit;
  }


# -----------------------------------------------------------------------------
# -- READ :
# -----------------------------------------------------------------------------

sub unix_time() {
    my $self = shift;
    return $self->{UNIXTIME}
  }

sub human_time() {
    my $self = shift;
    return $self->{HUMANTIME}
  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
